# Damien Leroy - UCLouvain
# June, 2008

import shutil, sys
from regexfile import regexfile

def main( src, dst, type, prefix, role=""):


    # Firewall file, output of iptables-save type
    def fw_iptables():
        print "parsing fw iptables..."
        
        # replacing the prefixes
        regexfile(src, '/tmp/fw1',
                      prefix.replace(".","\."), '$$'
                      )
        
        # add backets to line with prefixes
        regexfile( '/tmp/fw1', dst,
                      '(\-A[A-Za-z0-9\-_\. ,:/"]*\$[A-Za-z0-9\-_\. ,:/"\$]*)([\r\n]|$)',
                      r'[[\1]\\n]\2'
                      )
        
        
    # Bind main configuration file
    def ns_conf(): 
        print "parsing NS main configuration file for prefix '"+prefix+"'"
        
        
        # 1st : problematic query/transfer/notify-source
        regexfile(src, '/tmp/ns1' , '([\r\n]|^)([A-Za-z\-\t ]*-source[ \-A-Za-z6]*)'+
                prefix.replace(".","\.")+
                '([0-9a-f ;\.:]*)',
              r'\1[[\2$$\3]\\n]'
              )
        
        # 2nd : blocs using addresses

        regexfile('/tmp/ns1', '/tmp/ns2' ,
              '([\r\n]|^)([\t ]*server )'+ prefix.replace(".","\.")+
              '([0-9a-f\.: ]*\{*[A-Za-z0-9\-_\.,;:/"\s]*\};)',
              r'\1[[\2$$\3]\\n]'
              )
        
        

        # 3rd : address list
        regexfile('/tmp/ns2', '/tmp/ns3',
              '(!?)'+prefix.replace(".","\.")+'([0-9\.:]*[/0-9]* ?; ?)',
              r'[[\1$$\2]]'
              )
        
        # 4th : reverse blocs using addresses
        regexfile('/tmp/ns3', dst ,
              '([\r\n]|^)([\f\v\t ]*zone "[0-9a-fA-F:\.]*)'+
                prefix.replace(".","\.") +
              '([a-zA-Z\-\.]*"[\f\v\t ]*\{[\sA-Za-z0-9\-_\$\. ,;:/"]*\{'+
              '[\s\$\[\]A-Za-z0-9\-_\. ,;:/"]*\}[\sA-Za-z0-9\-_\.,;:/"]*\};)',
              r'\1[[\2$reverse:$\3]\\n]'
              )
        #(\{[A-Za-z0-9\-_\. ,;:/"\r\n\t]*\})'+
#              '?[A-Za-z0-9\-_\. ,;:/"\r\n\t]*

    # execute the function for the type given in arg
    try:
        locals()[type]()
        print "done"
    except KeyError :
        print 'Type of file not supported'
 

# return the string of "addr" in the reverse order used by dns
def reverse_addr(addr) : 
    
    output = ""
    parts = addr.split(":")
    
    if len (parts) > 1 : 
        # v6

        parts.reverse()
        for part in parts :
            
            for i in range(len(part),0,-1) :
                output += part[i-1]+"."
                
            for i in range(4-len(part)):
                output += "0."
            
    
    else : 
        #v4

        parts = addr.split(".")
        parts.reverse()
        for part in parts: 
            output += part+"."
            
    return output[0:-1]

if __name__ == '__main__':
    
    # if parameters are wrong, exit with error
    if len(sys.argv) != 6 and len(sys.argv) != 5:

        print '\nUsage:'
        print 'abstract.py src dst type_format prefix [role]'
        print 'type_format := fw_iptables,ns_conf'

        sys.exit(-1)

    else :  
        main(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])




